local MsgPacker = require("app.util.MsgPacker")

local MainScene = class("MainScene", function()
    return cc.Scene:create()
end)

function MainScene:ctor()
    display.loadSpriteFrames("common.plist", "common.png")

    --视频层
    live.VideoLayer:create():addTo(self)

    --向左滑动后消失层, 直播2.2版本后废弃
    self.scrollLayer = cc.ClippingRectangleNode:create(cc.rect(0, 0, display.width, display.height)) :addTo(self)

    --点赞层
    self.diggLayer = live.DiggLayer.new():addTo(self.scrollLayer)

    --用户进场层
    self.userEnterLayer = live.UserEnterLayer.new():addTo(self.scrollLayer)

    --小礼物动画层
    self.normalGiftLayer = live.NormalGiftLayer.new():addTo(self.scrollLayer)

    --弹幕层
    self.danmakuLayer = live.DanmakuLayer.new():addTo(self.scrollLayer)

    --特殊礼物图层
    self.specialGiftLayer = live.SpecialGiftLayer.new():addTo(self.scrollLayer)

    if not live.GestureGiftLayer then
        require "app.gift.special.gesture.GestureGiftLayer"
        require "app.gift.special.gesture.GestureGiftNode"
        require "app.gift.special.gesture.GestureGiftMakeLayer"
        require "app.gift.special.gesture.GestureGiftMakeSprite"
    end
    
    --播放手势礼物图层
    self.gestureGiftLayer = live.GestureGiftLayer.new():addTo(self.scrollLayer)

    --绘制手势礼物图层
    self.gestureGiftMakeLayer = live.GestureGiftMakeLayer.new():addTo(self.scrollLayer)

    local function tryPlayAnimation()
        self:tryPlayDiggAnimation()
        self:tryPlayUserEnterAnimation()
        self:tryPlayNormalAnimation()
        self:tryPlaySpecialAnimation()
        self:tryPlayDanmakuAnimation()
        self:tryPlayGestureAnimation()
    end
    cc.scheduler:scheduleScriptFunc(tryPlayAnimation, 0.2, false)

    self:registerScriptHandler(function(state)
        if state == "enter" then
            self:onEnter()
        end
    end)
end

--播放手势礼物动画
function MainScene:tryPlayGestureAnimation()
    if not live.messageManager.isGestureGiftMessageQueueEmpty or live.messageManager:isGestureGiftMessageQueueEmpty() then
        return
    end

    local msg = live.messageManager:getNextGestureGiftMessage()
    if msg then
        xpcall(function()
            local packedMsg = MsgPacker.packGestureGiftMsg(msg)
            local result = self.gestureGiftLayer:tryPlayAnimation(packedMsg)
            if result then
                live.messageManager:releaseNextGestureGiftMessage()
            end
        end,
        function(err)
            live.messageManager:releaseNextGestureGiftMessage()
            printLog("MainScene", "tryPlayDiggAnimation error:" .. err)
        end)
    end
end

--播放点赞动画
function MainScene:tryPlayDiggAnimation()
    if live.messageManager:isDiggMessageQueueEmpty() then
        return
    end

    local msg = live.messageManager:getNextDiggMessage()
    if msg then
        xpcall(function()
            local packedMsg = MsgPacker.packDiggMsg(msg)
            local result = self.diggLayer:tryPlayAnimation(packedMsg)
            if result then
                live.messageManager:releaseNextDiggMessage()
            end
        end,
        function(err)
            live.messageManager:releaseNextDiggMessage()
            printLog("MainScene", "tryPlayDiggAnimation error:" .. err)
        end)
    end
end

--播放用户进场动画
function MainScene:tryPlayUserEnterAnimation() 
    if live.messageManager:isUserEnterQueueEmpty() then 
        return
    end
    local msg = live.messageManager:getNextUserEnterMessage()
    if not msg then
        return
    end
    xpcall(function()
        local packedMsg = MsgPacker.packUserEnterMsg(msg)
        local result = self.userEnterLayer:tryPlayAnimation(packedMsg)
        if result then
            live.messageManager:releaseNextUserEnterMessage()
        end
    end,
    function(err)
        live.messageManager:releaseNextUserEnterMessage()
        printLog("MainScene", "tryPlayUserEnterAnimation error:" .. err)
    end)
end

--播放弹幕库礼物动画
function MainScene:tryPlayDanmakuAnimation()
    if live.messageManager:isDanmakuMessageQueueEmpty() then
        return
    end
    for i = live.Constant.DANMAKU_TRACK_COUNT - 1, 0, -1 do --这里是一次可以取两个消息
        local msg = live.messageManager:getNextDanmakuMessage(i)
        if  msg then
            xpcall(function()
                local packedMsg =  MsgPacker.packDanmakuMsg(msg)
                local result = self.danmakuLayer:tryPlayAnimation(packedMsg)
                if result then
                    live.messageManager:releaseNextDanmakuMessage(i)
                end
            end,
            function(err)
                live.messageManager:releaseNextDanmakuMessage(i)
                printLog("MainScene", "tryPlayDanmakuAnimation error:" .. err)
            end) 
        end
    end
end

--播放普通礼物动画
function MainScene:tryPlayNormalAnimation()
    if live.messageManager:isNormaGiftQueueEmpty() then
        return
    end
    for i = 0 , live.Constant.NORMAL_GIFT_CONTAINER_COUNT-1 do
        local msg = live.messageManager:getNormalGiftMessage(i)
        if not msg then
            return
        end
        xpcall(function()
            local packedMsg = MsgPacker.packNormalGiftMsg(msg)
            local result = self.normalGiftLayer:tryPlayAnimation(packedMsg)
            if result then
                live.messageManager:releaseNormalGiftMessage(i)
                --TODO: 丑陋的写法，后面改成可动态配置
                if packedMsg['giftId'] == 1 and packedMsg['repeatCount'] == 999 then
                    -- fake a special gift and insert to head of the queue
                    print(packedMsg['userAvatar'])
                    local strRose = textPack[nine_rose]
                    live.messageManager:insertSpecialGiftMessageAtFirst(1, 1000000, 1, 1, packedMsg['userName'], packedMsg['userAvatar'], 1, strRose, packedMsg.honorIcon)
                    print("send 999 gift")
                end
                if packedMsg['giftId'] == 2 and packedMsg['repeatCount'] == 99 then
                    -- fake a special gift and insert to head of the queue
                    print(packedMsg['userAvatar'])
                    local strRose = textPack[nine_beer]
                    live.messageManager:insertSpecialGiftMessageAtFirst(1, 1000001, 1, 1, packedMsg['userName'], packedMsg['userAvatar'], 1, strRose, packedMsg.honorIcon)
                    print("send 999 gift")
                end
            end
        end,
        function(err)
            live.messageManager:releaseNormalGiftMessage(i)
            printLog("MainScene", "tryPlayNormalAnimation error:" .. err)
        end)
    end
end

--播放特殊礼物动画
function MainScene:tryPlaySpecialAnimation()
    -- body
    if(live.messageManager:isSpecialGiftQueueEmpty() or self.specialGiftLayer.isPlaying) then
        return
    end
    local msg = live.messageManager:getNextSpecialGiftMessage()
    if msg then
        xpcall(function() 
            local packedMsg = MsgPacker.packSpecialGiftMsg(msg)
            self.specialGiftLayer:tryPlayAnimation(packedMsg)
        end,
        function(err)
            printLog("MainScene", "tryPlaySpecialAnimation error:" .. err)
            self.specialGiftLayer.isPlaying = false
        end)
        live.messageManager:releaseNextSpecialGiftMessage()
    end
end

--监听上层键盘收起事件，直播2.2版本后废弃
function MainScene:onKeyboardShow(height)
    local action= transition.sequence({
        cc.EaseIn:create(cc.MoveTo:create(0.25,cc.p(0,height)),1.0)
    })
    action:setTag(live.Constant.KEYBOARD_SHOW_ACTION)

    self.scrollLayer:stopActionByTag(live.Constant.KEYBOARD_SHOW_ACTION)
    self.scrollLayer:stopActionByTag(live.Constant.KEYBOARD_HIDE_ACTION)
    self.scrollLayer:runAction(action)
end

--监听上层键盘弹起事件，直播2.2版本后废弃
function MainScene:onKeyboardHide()
    local action = cc.Sequence:create({
        cc.EaseIn:create(cc.MoveTo:create(0.25,cc.p(0,0)),1.0)
    })
    action:setTag(live.Constant.KEYBOARD_HIDE_ACTION)

    self.scrollLayer:stopActionByTag(live.Constant.KEYBOARD_SHOW_ACTION)
    self.scrollLayer:stopActionByTag(live.Constant.KEYBOARD_HIDE_ACTION)
    self.scrollLayer:runAction(action)
end

--监听上层滑动事件，直播2.2版本后废弃
function MainScene:onScreenSwipeIn()
    local action = cc.Sequence:create({
        cc.CallFunc:create(function(el)
            el:setVisible(true)
        end),
        cc.EaseIn:create(cc.MoveTo:create(0.3,cc.p(0,0)),2.0)
    })
    self.scrollLayer:runAction(action)
end

--监听上层滑动事件，直播2.2版本后废弃
function MainScene:onScreenSwipeOut()
    local size = cc.Director:getInstance():getVisibleSize()
    local action = cc.Sequence:create({
        cc.EaseIn:create(cc.MoveTo:create(0.3,cc.p(size.width,0)),2.0),
        cc.CallFunc:create(function(el)
            el:setVisible(false)
        end)
    })
    self.scrollLayer:runAction(action)
end

--监听上层滑动事件，直播2.2版本后废弃
function MainScene:onScreenSwiping(distance)
    local size=cc.Director:getInstance():getVisibleSize()
    self.scrollLayer:setPositionX(distance)
    if(distance>=size.width) then
        self.scrollLayer:setVisible(false)
    else
        self.scrollLayer:setVisible(true)
    end
end

function MainScene:refreshWatermark(shortId)
    self.fixedLayer:refreshWatermark(shortId)
end

function MainScene:receiveGestureGiftMakePoint(image, x, y)
    self.gestureGiftMakeLayer:receiveGestureGiftMakePoint(image, x, y)
end

function MainScene:clearGestureGiftMake()
    self.gestureGiftMakeLayer:clearGestureGiftMake()
end

function MainScene:undoGestureGiftMake()
    self.gestureGiftMakeLayer:undoGestureGiftMake()
end

function MainScene:onEnter()
    print("********SearchPaths********")
    dump(cc.FileUtils:getInstance():getSearchPaths())
    print("********SearchResolutionsOrder********")
    dump(cc.FileUtils:getInstance():getSearchResolutionsOrder())
end

return MainScene
